/*
	os_dep.c
	
	OS dependent initialization routines.
	
	by Patrick C. Beard.
 */

#if defined(THINK_C)

#include <LowMem.h>
#include <console.h>
#include <signal.h>

#include "error.h"

// 256k of stack seems good. probably overkill.

#define cMinStackSpace (256L * 1024L)

static void SetMinimumStack(long minSize)
{
	long newApplLimit;

	if (minSize > LMGetDefltStack())
	{
		newApplLimit = (long) GetApplLimit() - (minSize - LMGetDefltStack());
		SetApplLimit((Ptr) newApplLimit);
		MaxApplZone();
	}
}

static void handle_interrupt(int sig)
{
	// reprime immediately.
	signal(SIGINT, &handle_interrupt);
	error("handle_interrupt: returning to top level", NULL);
}

void initialize_os()
{
	SetMinimumStack(cMinStackSpace);
	console_options.title = "\pMarlais Dylan";
	cshow(stdout);
	cgotoxy(1, 1, stdout);
	
	// install a signal handler to abort evaluation.
	signal(SIGINT, &handle_interrupt);
}

// simple getopt for Macintosh port. since Mac doesn't provide command line anyway.

char *optarg = 0;
int optind = 0;

int getopt(int argc, char *argv[], const char *options)
{
	char *arg;
	
	if (optind == 0) {
		optind = 1;
		optarg = argv[1];
	}
	
	return EOF;
}

#endif
