#include "screen.h"

short font=1;
short style = bold;
short pointsize = 24;

unsigned long delay=(5*60);				// delay between clock shifts, in ticks

ConstStr255Param title;					// title of program & window

static Boolean menubarhidden=FALSE;		// TRUE when main screen being saved

struct screen *screenlist;				// pointer to list of screens to save

extern void createmenus(void);			// in menus.c
extern void domenu(long);				// in menus.c
extern void doevent(EventRecord);		// in menus.c
extern Boolean idle(struct screen *);	// in trackidle.c
extern void MBarHide(void);				// in menu-bar.c
extern void MBarRestore(void);			// in menu-bar.c
extern void ReadPrefs(void);			// in prefs.c

/*
"Resume" routine in a System Error dialog
*/
pascal void bailout(void) {
	ExitToShell();
}

/*
Called when the screensaver activates
*/
void activate(struct screen *screen) {
	ObscureCursor();
	if (screen->hasMenuBar) {
		SelectWindow(screen->window);  // MBarHide needs frontmost window
		MBarHide();
		menubarhidden=TRUE;
		}

	ShowWindow(screen->window);
	screen->lastMoved = TickCount();
	return;
}

/*
Called when the screensaver deactivates
*/
void deactivate(struct screen *screen) {
	HideWindow(screen->window);
	if (screen->hasMenuBar) {
		MBarRestore();
		menubarhidden=FALSE;
		}

	return;
}

/*
Called whenever the machine is idle
*/
void savescreen(struct screen *screen) {
	unsigned long		timenumber;		// number of seconds since 1/1/1904
	Str255				timestring;		// written-out time of day
	short				size;			// length of the time string
	
	Rect r = screen->window->portRect;	// drawing rectangle of the window
	Point *p = &screen->p;				// location of clock
	Point				cursor;			// position of cursor

	if (screen->lastMoved + delay > TickCount()) return;
	
	SetPort(screen->window);
	TextFont(font);
	TextFace(style);
	TextMode(srcXor);
	TextSize(pointsize);

	EraseRect(&r);
	
	GetMouse(&cursor);
	LocalToGlobal(&cursor);
	if (PtInRect(cursor,&screen->globalRect))
		ObscureCursor();

	GetDateTime(&timenumber);
	IUTimeString(timenumber,FALSE,timestring);
	
	MoveTo(p->h,p->v);
	DrawString(timestring);
	size = StringWidth(timestring);
	
	screen->lastMoved = TickCount();
	
	p->v += pointsize;

	if (p->v < r.bottom) return;
	p->v = r.top + pointsize;
	
	p->h += size;
	if (p->h < r.right-size) return;
	p->h = r.left;
	return;
}

void main(void) {
	GDHandle			gdh;			// handle to graphics device
	struct screen		*screen;		// pointer to screen record
	Rect				r;
	PicHandle			ph;				// handle to "erase" picture
	StringHandle		h;				// handle to title string
	
	EventRecord			theEvent;
	
	short				size;			// width of title string
	
	ProcessSerialNumber	myProcessID,
						oldFrontProcessID,
						currentFrontProcessID;

	SysEnvRec			sysenvrec;		// for SysEnvirons	

	MaxApplZone();
/*
Initialize graphics
*/
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(bailout);
	InitCursor();

/*
Test for System 7 or later
*/
	SysEnvirons(1,&sysenvrec);
	if (sysenvrec.systemVersion < 0x700) {
		StopAlert(129,(ProcPtr)nil);
		ExitToShell();
		}

/*
Begin by getting the screensaver's own process serial number
*/
	GetCurrentProcess(&myProcessID);

/*
Get the title string of the program
*/
	h=GetString(128);
	HNoPurge((Handle)h);
	HLock((Handle)h);
	title=*h;
	
/*
Read the defaults
*/
	ReadPrefs();
	
/*
Set up the program's menus
*/
	createmenus();

/*
Get list of screens and create windows for them
*/
	if (sysenvrec.hasColorQD)
		for (gdh=GetDeviceList(); gdh; gdh=GetNextDevice(gdh)) {
			if (!(TestDeviceAttribute(gdh,screenDevice) && TestDeviceAttribute(gdh,screenActive)))
				continue;
			screen = (struct screen *)NewPtr(sizeof(struct screen));
			if (TestDeviceAttribute(gdh,mainScreen))
				screen->hasMenuBar = TRUE;
				else screen->hasMenuBar = FALSE;
			screen->globalRect = (*gdh)->gdRect;	// the whole screen
			
			screen->next = screenlist;
			screenlist = screen;
			}
		else {
			screenlist = (struct screen *)NewPtr(sizeof(struct screen));
			screenlist->hasMenuBar = TRUE;
			screenlist->globalRect = qd.screenBits.bounds;
			screenlist->next = (struct screen *)nil;
			}
			
	for (screen=screenlist; screen; screen=screen->next) {
		screen->idleSince = TickCount();
		screen->beingSaved = FALSE;
		screen->window = NewWindow(
						&(screen->windowRec),	// storage
						&(screen->globalRect),	// fill whole screen
						title,					// title
						FALSE,					// originally hidden
						plainDBox,				// appearance (plain)
						(WindowPtr)nil,			// behind no other window
						FALSE,					// no go-away box
						0L	);					// nothing in the RefCon

		SetPort(screen->window);
		BackPat(qd.black);					// make the default background black
		r=screen->window->portRect;
		ph=OpenPicture(&r);
		EraseRect(&r);
		TextFont(font);
		TextFace(style);
		TextMode(srcXor);
		TextSize(pointsize);
		size=StringWidth(title);
		MoveTo((r.right-r.left-size)/2,(r.bottom-r.top)/2);
		DrawString(title);
		ClosePicture();
		SetWindowPic(screen->window,ph);
		
		screen->p.h = r.left;
		screen->p.v = r.top + pointsize;
	}
	
/*
The Main Event!
*/
	FlushEvents(everyEvent,0);

	while(1) {
	
	SystemTask();	
	if (GetNextEvent(activMask|mDownMask|keyDownMask|autoKeyMask|osMask,&theEvent))
	switch (theEvent.what) {
// Update events aren't handled because the windows have auto-update PICTs
	case osEvt:
		if (theEvent.message>>24!=suspendResumeMessage) {
			break;
			}
		if (theEvent.message & resumeFlag) {
			InitCursor();
			if (menubarhidden) MBarHide();
			}
		else {
			if (menubarhidden) MBarRestore();
			}
		break;
	default:
		doevent(theEvent);				// menu events handled elsewhere
	};

	for (screen=screenlist; screen; screen=screen->next) {
		Point cursor;
		GetMouse(&cursor);
		LocalToGlobal(&cursor);
		if (idle(screen)) {
			if (screen->beingSaved == FALSE) {
				screen->beingSaved = TRUE;
				activate(screen);
				if (screen->hasMenuBar) {
					GetFrontProcess(&oldFrontProcessID);
					SetFrontProcess(&myProcessID);
					} // if hasMenuBar
				} // if beingSaved
			savescreen(screen);
			} // if idle
		else if (screen->beingSaved == TRUE) {
				Boolean AmFrontProcess;
				deactivate(screen);
				if (screen->hasMenuBar) {
					GetFrontProcess(&currentFrontProcessID);
					SameProcess(&currentFrontProcessID,&myProcessID,&AmFrontProcess);
					if (AmFrontProcess)
						SetFrontProcess(&oldFrontProcessID);
					} // if hasMenuBar
				screen->beingSaved = FALSE;
			} // else if beingSaved
		} // for screen
	} // while 1
} // main()

